#include <iostream>
#include <queue>
#include <utility>
#include <algorithm>
#include <iomanip>

using namespace std;

struct node {
    int first_color, last_color, groups;
    node (int color = -1)
    {
        this->first_color = color;
        this->last_color = color;
        this->groups = 1;
    }
    node (int f, int l, int g)
    {
        this->first_color = f;
        this->last_color = l;
        this->groups = g;
    }
};

struct tree_t {
private:
    node* tree;
    int n;

    void merge (int i)
    {
        int left = 2 * i, right = 2 * i + 1;
        if (tree [left].first_color == -1 and tree [left].last_color == -1 and tree [left].groups == -1)
        {
            tree [i] = node (-1, -1, -1);
        }
        else if (tree [right].first_color == -1 and tree [right].last_color == -1 and tree [right].groups == -1)
        {
            tree [i] = tree [left];
        }
        else
        {
            tree [i] = node (tree [left].first_color,
                             tree [right].last_color,
                             tree [left].groups + tree [right].groups
                                - (tree [left].last_color == tree [right].first_color));
        }
    }

public:
    tree_t (int* arr, int len)
    {
        n = 1;
        while (n <= len) n *= 2;
        //cout << "Last row length: " << n << endl;
        tree = new node [2 * n];
        for (int i = 0 ; i < n ; i ++)
        {
            if (i >= len) tree [i + n] = node (-1, -1, -1);
            else tree [i + n] = node (arr [i]);
        }
        for (int i = n - 1 ; i > 0 ; i --)
        {
            merge (i);
        }
    }
    ~tree_t ()
    {
        delete[] tree;
    }

    void replace (int i, int color)
    {
        tree [n + i] = node (color);
        int pos = (n + i) / 2;
        while (pos > 0)
        {
            merge (pos);
            pos /= 2;
        }
    }

    void replace (int i, node group)
    {
        tree [n + i] = group;
        int pos = (n + i) / 2;
        while (pos > 0)
        {
            merge (pos);
            pos /= 2;
        }
    }

    int query ()
    {
        return tree [1].groups;
    }

    void remove (int i)
    {
        for ( ; i < n - 1 ; i ++)
            replace (i, tree [n + i + 1]);
    }

    void print ()
    {
        cout << "N = " << this->n << "\n";
        cout << "------------------------\n";
        int level = 1;
        for (int i = 1 ; i < 2 * n ; i ++)
        {
            cout << "(" << tree [i].first_color << "," << tree [i].last_color << "," << tree [i].groups << ") ";
            if (level == i)
            {
                level += level + 1;
                cout << "\n";
            }
        }
        cout << "\n------------------------\n";
    }

    void insert (int i, int color)
    {
        for (int j = n - 1 ; j > i ; j --)
        {
            //cout << "replace " << j << " w/ " << j + 1 << endl;
            //cout << "(" << tree [j].first_color << "," << tree [j].last_color << "," << tree [j].groups << ") -> "
            //     << "(" << tree [j + 1].first_color << "," << tree [j + 1].last_color << "," << tree [j + 1].groups << ")"
            //     << endl;

            replace (j, tree [n + j - 1]);
        }
        replace (i, node (color));
    }

};

int main ()
{
	cin.tie(nullptr);
	ios::sync_with_stdio(false);
    int n;
    cin >> n;
    int* arr = new int [n];
    for (int i = 0 ; i < n ; i ++)
        cin >> arr [i];
    tree_t tree = tree_t (arr, n);
    //tree.print ();
    int m;
    cin >> m;
    for (int i = 0 ; i < m ; i ++)
    {
        int q;
        cin >> q;
        if (q == 1)
        {
            int pos;
            cin >> pos;
            pos --;
            tree.remove(pos);
        }
        else if (q == 2)
        {
            int pos, color;
            cin >> pos >> color;
            pos --;
            //tree.insert(pos, color);
        }
        else if (q == 3)
        {
            int pos, color;
            cin >> pos >> color;
            pos --;
            tree.replace(pos, color);
        }
        else if (q == 4)
        {
            cout << tree.query() << "\n";
        }
        //tree.print();
    }
    return 0;
}
/**
10
2 3 3 1 1 4 5 1 1 3
8
1 6
3 6 1
4
1 1
2 2 1
4
2 9 2
4


10
2 3 3 1 1 4 5 1 1 3
16
3 1 3
4
3 6 1
4
3 7 3
4
3 1 1
4
3 2 1
4
3 3 1
4
3 7 1
4
3 10 1
4

*/


