#include <cstdio>
#include <random>

using namespace std;

//mt19937 randomGen('B'+'o'+'r'+'b'+'i');
mt19937 randomGen(2708199);

struct Treap
{
    Treap *left, *right;
    int key;
    int priority;
    int cnt;

    Treap(int _key)
    {
        key = _key;
        priority = randomGen();
        left = right = nullptr;
        cnt = 1;
    }

    void recalc()
    {
        cnt = 1;
        if(left != nullptr)
        {
            cnt += left->cnt;
        }
        if(right != nullptr)
        {
            cnt += right->cnt;
        }
    }
};

Treap* merge(Treap* sm, Treap* bg)
{
    if(sm == nullptr)
    {
        return bg;
    }
    if(bg == nullptr)
    {
        return sm;
    }
    if(sm->priority > bg->priority)
    {
        sm->right = merge(sm->right, bg);
        sm->recalc();
        return sm;
    }
    else
    {
        bg->left = merge(sm, bg->left);
        bg->recalc();
        return bg;
    }
}

int get_cnt(Treap* t)
{
    if(t == nullptr)
    {
        return 0;
    }
    return t->cnt;
}

pair <Treap*, Treap*> split(Treap* t, int key)
{
    if(t == nullptr)
    {
        return {nullptr, nullptr};
    }
    int pos = get_cnt(t->left) + 1;
    if(pos < key)
    {
        auto splitResults = split(t->right, key - pos);
        t->right = splitResults.first;
        t->recalc();
        return {t, splitResults.second};
    }
    else
    {
        auto splitResults = split(t->left, key);
        t->left = splitResults.second;
        t->recalc();
        return {splitResults.first, t};
    }
}

Treap* root = nullptr;

void firstAdd(int key)
{
    if(root == nullptr)
    {
        root = new Treap(key);
        return;
    }
    Treap* t = new Treap(key);
    root = merge(root, t);
}

int get_from_left(Treap* t)
{
    if(t == nullptr)
    {
        return 0;
    }
    if(t->right == nullptr)
    {
        return t->key;
    }
    return get_from_left(t->right);
}

int get_from_right(Treap* t)
{
    if(t == nullptr)
    {
        return 0;
    }
    if(t->left == nullptr)
    {
        return t->key;
    }
    return get_from_right(t->left);
}

void lkd(Treap* t)
{
    if(t == nullptr)
    {
        return;
    }
    lkd(t->left);
    printf("%d ", t->key);
    lkd(t->right);
}

int n;
int br;

void rem(int index)
{
    auto splitResults = split(root, index);
    auto splitResults2 = split(splitResults.second, 2);
    //lkd(splitResults2.second);
    //printf("%d\n", splitResults2.first->key);
    int leftOne = get_from_left(splitResults.first);
    int rightOne = get_from_right(splitResults2.second);
    if(splitResults2.first == nullptr) return;
    int temp = splitResults2.first->key;
    if(leftOne != temp && rightOne != temp)
    {
        if(leftOne == rightOne)
        {
            br -= 2;
        }
        else
        {
            br--;
        }
    }
    //printf("%d %d\n", leftOne, rightOne);
    root = merge(splitResults.first, splitResults2.second);
}

void add(int index, int key)
{
    auto splitResults = split(root, index);
    int leftOne = get_from_left(splitResults.first);
    int rightOne = get_from_right(splitResults.second);
    if(leftOne != key && rightOne != key)
    {
        br++;
        if(leftOne == rightOne)
        {
            br++;
        }
    }
    Treap* t = new Treap(key);
    root = merge(merge(splitResults.first, t), splitResults.second);
}

void solve()
{
    int q;
    scanf("%d", &q);
    int type, pos, color;
    while(q--)
    {
        scanf("%d", &type);
        if(type == 1)
        {
            scanf("%d", &pos);
            rem(pos);
        }
        else if(type == 2)
        {
            scanf("%d %d", &pos, &color);
            add(pos, color);
        }
        else if(type == 3)
        {
            scanf("%d %d", &pos, &color);
            rem(pos);
            add(pos, color);
        }
        else
        {
            printf("%d\n", br);
        }
        //lkd(root);
        //printf("\n");
    }
}

int main()
{
    scanf("%d", &n);
    int v;
    int prev = 0;
    for(int i = 0; i < n; i++)
    {
        scanf("%d", &v);
        firstAdd(v);
        if(prev != v)
        {
            br++;
        }
        prev = v;
    }
    //rem(1);
    //printf("%d\n", br);
    solve();

    return 0;
}

/*
10
2 3 3 1 1 4 5 1 1 3
8
1 6
3 6 1
4
1 1
2 2 1
4
2 9 2
4
*/
